@@ -1,7 +1,8 @@
 import sys
-from typing import List
+from typing import List, Optional, Callable, Any, Iterable, Mapping
 
 import global_val
+from storages.proxy_dict import ProxyDict
 
 sys.path.append("PaperCrawlerUtil")
 sys.path.append("PaperCrawlerUtil/proxypool")
@@ -27,29 +28,12 @@ from proxypool.processors.getter import Getter
 from proxypool.processors.server import app
 from proxypool.processors.tester import Tester
 from global_val import *
+from constant import *
+
 
 PROXY_POOL_URL = ""
 logging.basicConfig(filename='crawler_util.log', level=logging.WARNING)
-log_style = "log"
-HTTP = "http://"
-COLON_SEPARATOR = ":"
-
-EQUAL = "equal"
-NOT_EQUAL = "not equal"
-IN = "in"
-NOT_IN = "not in"
-LESS_THAN = "less than"
-MORE_THAN = "more than"
-GREATER_AND_EQUAL = "greater and equal"
-LESS_THAN_AND_EQUAL = "less than and equal"
-
-LOG_STYLE_LOG = "log"
-LOG_STYLE_PRINT = "print"
-LOG_STYLE_ALL = "all"
-
-NEED_CRAWLER_LOG = False
-NEED_COMMON_LOG = False
-NEED_DOCUMENT_LOG = False
+log_style = LOG_STYLE_PRINT
 
 PROXY_POOL_CAN_RUN_FLAG = True
 
@@ -65,7 +49,24 @@ def log(string: str) -> None:
         print(string)
 
 
-class ThreadGetter(threading.Thread):
+class CanStopThread(threading.Thread):
+
+    def __init__(self) -> None:
+        super().__init__()
+
+    def stop(self):
+        raise ThreadStopException()
+
+    def save_dict(self) -> bool:
+        if global_val.get_value(STORAGE_CONF) == STORAGE_REDIS:
+            return True
+        else:
+            log("保存dict")
+            d = ProxyDict()
+            return d.save()
+
+
+class ThreadGetter(CanStopThread):
     def __init__(self, redis_host, redis_port, redis_password, redis_database, storage, need_log: bool = True):
         threading.Thread.__init__(self)
         self.need_log = need_log
@@ -82,7 +83,7 @@ class ThreadGetter(threading.Thread):
                need_log=self.need_log, storage=self.storage).run()
 
 
-class ThreadTester(threading.Thread):
+class ThreadTester(CanStopThread):
     def __init__(self, redis_host, redis_port, redis_password, redis_database, storage, need_log: bool = True):
         threading.Thread.__init__(self)
         self.need_log = need_log
@@ -99,7 +100,7 @@ class ThreadTester(threading.Thread):
                need_log=self.need_log, storage=self.storage).run()
 
 
-class ThreadServer(threading.Thread):
+class ThreadServer(CanStopThread):
     def __init__(self, host, port, threaded):
         threading.Thread.__init__(self)
         self.host = host
@@ -156,6 +157,19 @@ def is_ip(proxy_test: str = "") -> bool:
     return flag
 
 
+def stop_thread(thread_list: List[CanStopThread] = []) -> int:
+    count = 0
+    for t in thread_list:
+        try:
+            t.stop()
+        except ThreadStopException as e:
+            log("线程{}结束".format(t.name))
+            count = count + 1
+        except Exception as e:
+            log("结束线程{}错误{}".format(t.name, e))
+    return count
+
+
 def basic_config(log_file_name: str = "crawler_util.log",
                  log_level=logging.WARNING,
                  proxy_pool_url: str = "",
@@ -170,8 +184,22 @@ def basic_config(log_file_name: str = "crawler_util.log",
                  need_storage_log: bool = True,
                  api_host: str = "127.0.0.1",
                  api_port: int = 5555,
-                 proxypool_storage: str = "redis") -> None:
+                 proxypool_storage: str = STORAGE_REDIS,
+                 set_daemon: bool = True,
+                 proxy_score_max: int = 100,
+                 proxy_score_min: int = 0,
+                 proxy_score_init: int = 10,
+                 proxy_number_max: int = 50000,
+                 proxy_number_min: int = 0,
+                 dict_store_path: str = "dict.db") -> tuple:
     """
+    :param dict_store_path: 选择字典方式存储时，最后文件保存的地址（同时也是加载地址）
+    :param proxy_number_min: 最小池容量
+    :param proxy_number_max: 最大池容量
+    :param proxy_score_init: proxy评分的初始值，添加的时候自动赋值为该值
+    :param proxy_score_min: proxy评分的最小值，小于此值时，丢弃
+    :param proxy_score_max: proxy评分的最大值，测试可用时，赋值为该值
+    :param set_daemon: 设置是否需要守护线程，即主线程结束，子线程也结束
     :param need_storage_log: 是否需要存储模块（redis)等的日志信息（不影响重要信息输出）
     :param proxypool_storage:代理池的存储方式，可以选择redis或者dict
     :param api_port: FLASK端口
@@ -190,23 +218,31 @@ def basic_config(log_file_name: str = "crawler_util.log",
     :return:
     """
     global PROXY_POOL_URL, PROXY_POOL_CAN_RUN_FLAG
-    global NEED_CRAWLER_LOG, NEED_COMMON_LOG, NEED_DOCUMENT_LOG
     global log_style
-    set_cross_file_variable([("REDIS", (redis_host, redis_port, redis_password, redis_database)),
-                             ("storage", proxypool_storage), ("global_dict", {}),
-                             ("storage_log", need_storage_log), ("getter_log", need_getter_log),
-                             ("tester_log", need_tester_log)])
+    set_cross_file_variable([(REDIS_CONF, (redis_host, redis_port, redis_password, redis_database)),
+                             (STORAGE_CONF, proxypool_storage), (CROSS_FILE_GLOBAL_DICT_CONF, {}),
+                             (STORAGE_LOG_CONF, need_storage_log), (GETTER_LOG_CONF, need_getter_log),
+                             (TESTER_LOG_CONF, need_tester_log), (PROXY_SCORE_MAX, proxy_score_max),
+                             (PROXY_SCORE_MIN, proxy_score_min), (PROXY_SCORE_INIT, proxy_score_init),
+                             (POOL_MAX, proxy_number_max), (POOL_MIN, proxy_number_min),
+                             (DICT_STORE_PATH, dict_store_path)])
     PROXY_POOL_URL = proxy_pool_url
     log_style = logs_style
     if require_proxy_pool and PROXY_POOL_CAN_RUN_FLAG and len(
             redis_host) > 0 and 0 <= redis_database <= 65535 and (
             0 <= redis_port <= 65535) and len(api_host) > 0 and (65535 >= api_port >= 0):
+        s = None
+        t = None
+        g = None
         try:
             g = ThreadGetter(redis_host=redis_host, redis_port=redis_port, redis_password=redis_password,
                              redis_database=redis_database, need_log=need_getter_log, storage=proxypool_storage)
             t = ThreadTester(redis_host=redis_host, redis_port=redis_port, redis_password=redis_password,
                              redis_database=redis_database, need_log=need_tester_log, storage=proxypool_storage)
             s = ThreadServer(host=api_host, port=api_port, threaded=True)
+            g.setDaemon(set_daemon)
+            t.setDaemon(set_daemon)
+            s.setDaemon(set_daemon)
             g.start()
             t.start()
             s.start()
@@ -228,9 +264,17 @@ def basic_config(log_file_name: str = "crawler_util.log",
             PROXY_POOL_URL = url
         log("启动proxypool完成")
         PROXY_POOL_CAN_RUN_FLAG = False
+        return s, g, t
     else:
-        if require_proxy_pool:
+        if require_proxy_pool and PROXY_POOL_CAN_RUN_FLAG:
             log("redis或者Flask配置错误")
+            return ()
+        elif require_proxy_pool and not PROXY_POOL_CAN_RUN_FLAG:
+            log("无法重复启动proxypool")
+            return ()
+        elif not require_proxy_pool:
+            return ()
+        return ()
 
 
 def get_split(lens: int = 20, style: str = '=') -> str:
@@ -278,10 +322,12 @@ def two_one_choose(p: int = 0.5) -> bool:
         return False
 
 
-def local_path_generate(folder_name: str, file_name: str = "", suffix: str = ".pdf") -> str:
+def local_path_generate(folder_name: str, file_name: str = "",
+                        suffix: str = ".pdf", need_log: bool = True) -> str:
     """
     create a folder whose name is folder_name in folder which code in if folder_name
     is not exist. and then concat pdf_name to create file path.
+    :param need_log: 是否需要日志，不影响重要信息输出
     :param suffix: 自动命名时文件后缀
     :param folder_name: 待创建的文件夹名称
     :param file_name: 文件名称，带后缀格式
@@ -291,7 +337,7 @@ def local_path_generate(folder_name: str, file_name: str = "", suffix: str = ".p
         folder_name = os.path.abspath(".")
     try:
         if os.path.exists(folder_name):
-            if NEED_COMMON_LOG:
+            if need_log:
                 log("文件夹{}存在".format(folder_name))
         else:
             os.makedirs(folder_name)
@@ -359,7 +405,29 @@ class NoProxyException(Exception):
     def __repr__(self) -> str:
         return super().__repr__()
 
+
+class ThreadStopException(Exception):
+
+    def __init__(self, *args: object) -> None:
+        super().__init__(*args)
+
+    def __str__(self) -> str:
+        return repr("停止线程")
+
+    def __repr__(self) -> str:
+        return super().__repr__()
+
+
 # if __name__ == "__main__":
-#     pool = ProxyPool(thread_id="001", thread_name="proxypool")
-#     pool.start()
-#     print("123")
+#     a = CanStopThread()
+#     basic_config(logs_style=LOG_STYLE_PRINT)
+#     try:
+#         a.stop()
+#     except ThreadStopException as e:
+#         log("线程{}结束".format(a.name))
+#     except Exception as e:
+#         log("结束线程{}错误{}".format(a.name, e))
+#     except BaseException as e:
+#         log(e)
+
+
